require 'cor/version'
require 'fileutils'

module Cor
  Project_Types = %w(game app application library exp experimentation editor)
  Platform_Types = %w(Android Windows iOS)

  class Executor
    def self.version(args)
      puts "Code on Rmake gem Version #{Cor::VERSION}"
    end

    def self.call_command(args)
      command = self.analyze_argument(args)

      if command[:command].to_s == ''
        command[:command] = :help
      end

      if self.methods(false).include?(command[:command])
        self.send(command[:command], command[:args])
      else
        puts "undefined command #{args[0]}"
        puts "ARGS: #{args.join(', ')}"
      end
    end

    def self.analyze_argument(args)
      command = args.shift.to_s

      result = {
          :command => command.gsub(':', '_').to_sym,
          :args => args
      }

      result
    end

    def self.init(args)
      FileUtils.mkdir_p(%w(./bin ./projects ./materials ./packages))
      unless File.exists?('./rmake.yml')
        source_root = File.expand_path('..', File.dirname(__FILE__))
        FileUtils.cp("#{source_root}/template_rmake.yml", './rmake.yml')
      end
    end

    def self.project_create(args)
      project_type = args[0]
      project_name = args[1].to_s

      unless Cor::Project_Types.include? project_type
        puts "Wrong Project Type #{project_type}"
        return
      end

      if project_name =~  /\w+/
        project_dir = "./projects/#{project_name}"
        if !File.exists?(project_dir)
          FileUtils.mkdir_p(%W(#{project_dir} #{project_dir}/materials #{project_dir}/data #{project_dir}/src))
        else
          puts "#{project_name} Project is already exists. "
        end
      else
        puts "Wrong Project Name #{project_name}. You can use [a-zA-Z_]."
      end
    end

    def self.project_packaging(args)

    end

    def self.project_checkout(args)

    end

    def self.project_upload(args)

    end

    def self.project_release(args)

    end

    def self.install(args)

    end

    def self.update(args)

    end

    def self.help(args)
      puts <<-EOS
Cor gem provides convenient commands for Code on Rmake.

  Usage:
    cor help
    cor version
    cor command [arguments...]

  Commands:
    cor version
    cor project:create my_game_directory

  Information:
    https://github.com/akasata/cor-gem

  About Code on Rmake:
    https://core.rmake.jp/

EOS

    end


  end
end
